#include <fork.h>
#include  <io.h>

main(void) {
	int i;

	for(i=0;i<async_groupsize();i++)
		if ($ == 0)  
			pprintf("iteration i = %d, asyncgroupsize = %d \n",i, async_groupsize());
	barrier;
}
/*

Mon Mar 22 10:41:29 GMT 1999

program output:

#0000# iteration i = 0, asyncgroupsize = 4 

Christoph comments:

Dear Casiano,

groupsize() and async_groupsize() just inspect the contents of the
current group's synchronization cell.
As the conditional is in an asynchronous region, it cannot change 
(split) the group (here, the root group).
But, since barrier also affects (decrements) the synchronization cell,
the "fast" processors $=1,2,... that do not have to work in a lengthy
printf() call execute the iterations of the i loop very fast and then
decrement the synchronization cell. It happens that the second argument 
of the printf() call is evaluated by processor 0 just before the other 
processors come to execute the barrier, this explains the result 4.
When processor 0 comes to finish its first iteration, increments i and
checks the exit condition, it finds a 1 in the synchronization cell 
that is returned by async_groupsize(), and hence exits the loop.

There are several possibilities to modify this program accordingly:

(1) use __STARTED_PROCS__ instead of async_groupsize().
    This value is a run-time constant (-> no race conditions).

(2) use synchronous mode and a shared loop variable.
    This will put an implicit barrier between iterations.
    Note that (async_)groupsize() relates the group being active
    when the call is performed. In the body of a shared loop
    with a private exit condition it relates to the subgroup
    of iterating processors, not to the parent group.

(3) evaluate async_groupsize() to a temporary variable BEFORE the loop
    and use that temporary in the loop exit condition. Race conditions
    are possible nevertheless if processors may become asynchronous 
    before this assignment.


Best regards,

--- Christoph

*/
